package com.adc.da.util.persistence;


import com.adc.da.util.constant.DeleteFlagEnum;
import com.adc.da.util.http.PageInfo;
import com.adc.da.util.persistence.entity.TreeEntity;
import com.adc.da.util.utils.CollectionUtils;
import com.adc.da.util.utils.PersistenceUtils;
import com.adc.da.util.utils.StringUtils;
import com.adc.da.util.utils.UUID;

import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.criterion.CriteriaSpecification;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.jpa.repository.support.JpaEntityInformation;
import org.springframework.data.jpa.repository.support.SimpleJpaRepository;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

public class BaseRepositoryImpl<T, ID extends Serializable> extends SimpleJpaRepository<T, ID> implements IBaseRepository<T, ID> {

    private static final Logger logger = LoggerFactory.getLogger(BaseRepositoryImpl.class);
    protected final EntityManager entityManager;
    protected final JpaEntityInformation jpaEntityInformation;

    public BaseRepositoryImpl(JpaEntityInformation entityInformation, EntityManager entityManager) {
        super(entityInformation, entityManager);
        this.jpaEntityInformation = entityInformation;
        this.entityManager = entityManager;
    }

    @Override
    public T saveTreeEntity(T entity) {
        TreeEntity treeEntity = (TreeEntity) entity;
        if (StringUtils.isEmpty(treeEntity.getId())) {
            treeEntity.setId(UUID.randomUUID());
        }

        String oldParentIds = treeEntity.getParentIds();
        TreeEntity parent = treeEntity.getParent();
        if (parent != null) {
            treeEntity.setParentId(parent.getId());
            treeEntity.setParentIds(parent.getParentIds() + parent.getId() + ",");
        }

        if (StringUtils.isNotEmpty(oldParentIds)) {
            List<TreeEntity> treeEntityList = list("from " + entity.getClass().getSimpleName() + " where parent_ids like ?", "%," + treeEntity.getId() + ",%");
            if (CollectionUtils.isNotEmpty(treeEntityList)) {
                for (TreeEntity childTreeEntity : treeEntityList) {
                    childTreeEntity.setParentIds(childTreeEntity.getParentIds().replace(oldParentIds, treeEntity.getParentIds()));
                    this.save((T) childTreeEntity);
                }
            }
        }
        return (T) this.save(entity);
    }

    @Override
    public void deleteLogic(ID id) {
        String hql = "update " + jpaEntityInformation.getEntityName() + " set delFlag = '" + DeleteFlagEnum.DELETE.getValue() + "' where id = ?1";
        executeUpdate(hql, id);
    }

    @Override
    public void deleteLogicInBatch(List<String> ids) {
        String hql = "update " + jpaEntityInformation.getEntityName() + " set delFlag = '" + DeleteFlagEnum.DELETE.getValue() + "' where id in (:ids)";
        Query query = entityManager.createQuery(hql);
        query.setParameter("ids", ids);
        query.executeUpdate();
    }

    @Override
    public void deleteTreeEntity(ID id) {
        T t = getOne(id);
        if (t != null && t instanceof TreeEntity) {
            TreeEntity treeEntity = (TreeEntity) t;
            treeEntity.setDelFlag(DeleteFlagEnum.DELETE.getValue());
            save((T) treeEntity);

            String hql = "update " + jpaEntityInformation.getEntityName() + " set delFlag = '" + DeleteFlagEnum.DELETE.getValue() + "' where parentIds like ?";
            executeUpdate(hql, "%" + treeEntity.getId() + ",%");
        }
    }

    @Override
    public T getByHql(String hql, Object... params) {
        Query query = getQueryWithParams(hql, params);
        return getFirst(query);
    }

    @Override
    public T getByHql(String hql, Map<String, Object> params) {
        Query query = getQueryWithParamMap(hql, params);
        return getFirst(query);
    }

    @Override
    public <T> List<T> list(String hql, Object... params) {
        Query query = getQueryWithParams(hql, params);
        return query.getResultList();
    }

    @Override
    public <T> List<T> list(String hql, Map<String, Object> params) {
        Query query = getQueryWithParamMap(hql, params);
        return query.getResultList();
    }

    @Override
    public <T> List<T> findAllWithoutDeleted() {
        return list("from " + jpaEntityInformation.getEntityName() + " where delFlag = 0");
    }

    @Override
    public PageInfo<T> page(PageInfo pageInfo, String hql, Object... params) {
        // 查询数据，统计条数
        Query query = getQueryWithParams(hql, params);
        query.setFirstResult((pageInfo.getPageNo() - 1) * pageInfo.getPageSize());
        query.setMaxResults(pageInfo.getPageSize());
        List<T> resultList = query.getResultList();

        Query pageQuery = getQueryWithParams(buildCountHQL(hql), params);
        long count = (long) pageQuery.getSingleResult();

        pageInfo.setList(resultList);
        pageInfo.setCount(count);
        return pageInfo;
    }

    @Override
    public PageInfo<T> page(PageInfo pageInfo, String hql, Map<String, Object> params) {
        // 查询数据，统计条数
        Query query = getQueryWithParamMap(hql, params);
        query.setFirstResult((pageInfo.getPageNo() - 1) * pageInfo.getPageSize());
        query.setMaxResults(pageInfo.getPageSize());
        List<T> resultList = query.getResultList();

        Query pageQuery = getQueryWithParamMap(buildCountHQL(hql), params);
        long count = (long) pageQuery.getSingleResult();

        pageInfo.setList(resultList);
        pageInfo.setCount(count);
        return pageInfo;
    }

    @Override
    public int executeUpdate(String hql, Object... params) {
        Query query = getQueryWithParams(hql, params);
        return query.executeUpdate();
    }

    private static String buildCountHQL(String hql) {
        StringBuilder countHql = new StringBuilder();
        if (hql.trim().startsWith("select")) {
            int fromIndex = hql.indexOf("from");
            int selectIndex = hql.indexOf("select");
            String hqlBetweenSelectAndForm = hql.substring(selectIndex + 6, fromIndex);
            String hqlNoSelect = hql.substring(fromIndex, hql.length());
            countHql.append(" select count(").append(hqlBetweenSelectAndForm).append(") ").append(hqlNoSelect);
        } else if (hql.trim().startsWith("from")) {
            countHql.append(" select count(*) ").append(hql);
        }
        return countHql.toString();
    }

    private Query getQueryWithParamMap(String hql, Map<String, Object> params) {
        logger.info("HQL [{}]", hql);
        if(CollectionUtils.isNotEmpty(params)) {
            logger.info("HQL Param: {}", params);
            logger.info("HQL Filled: [{}]", PersistenceUtils.fillParams(hql, params));
        }

        Query query = entityManager.createQuery(hql);
        if (CollectionUtils.isNotEmpty(params)) {
            for (String key : params.keySet()) {
                Object value = params.get(key);
                query.setParameter(key, value);
            }
        }
        return query;
    }

    private Query getQueryWithParams(String hql, Object[] params) {
        logger.info("HQL [{}]", hql);
        if(params != null && params.length > 0) {
            logger.info("HQL Param: {}", Arrays.asList(params));
            logger.info("HQL Filled: [{}]", PersistenceUtils.fillParams(hql, params));
        }

        Query query = entityManager.createQuery(hql);
        int index = 1;
        for (Object param : params) {
            query.setParameter(index, param);
            index++;
        }
        return query;
    }

    private T getFirst(Query query) {
        T obj = null;
        List<T> resultList = query.getResultList();
        if (CollectionUtils.isNotEmpty(resultList)) {
            obj = resultList.get(0);
        }
        return obj;
    }

    /**
     * 以下方法为对Native Query的支持...
     */
    private Query getNativeQueryWithParams(String sql, Class resultClass, Object[] params) {
        logger.info("SQL [{}]", sql);
        if(params != null && params.length > 0) {
            logger.info("SQL Param: {}", Arrays.asList(params));
            logger.info("SQL Filled: [{}]", PersistenceUtils.fillParams(sql, params));
        }
        Query query;
        if (resultClass == null) {
        	query = entityManager.createNativeQuery(sql);
        } else {
        	query = entityManager.createNativeQuery(sql, resultClass);
        }
        int index = 1;
        for (Object param : params) {
            query.setParameter(index, param);
            index++;
        }
        return query;
    }
    
    private Query getNativeQueryWithParamMap(String sql, Class resultClass, Map<String, Object> params) {
        logger.info("SQL [{}]", sql);
        if(CollectionUtils.isNotEmpty(params)) {
            logger.info("SQL Param: {}", params);
            logger.info("SQL Filled: [{}]", PersistenceUtils.fillParams(sql, params));
        }
        Query query;
        if (resultClass == null) {
        	query = entityManager.createNativeQuery(sql);
        } else {
        	query = entityManager.createNativeQuery(sql, resultClass);
        }
        if (CollectionUtils.isNotEmpty(params)) {
            for (String key : params.keySet()) {
                Object value = params.get(key);
                query.setParameter(key, value);
            }
        }
        return query;
    }

    /**
     * 执行Native Query修改语句
     */
    @Override
    public int executeNativeUpdate(String sql, Object... params) {
        Query query = getNativeQueryWithParams(sql, null, params);
        return query.executeUpdate();
    }

    /**
     * Native Query获取单条数据
     */
    @Override
    public T getBySql(String sql, Class<T> resultClass, Object... params) {
        Query query = getNativeQueryWithParams(sql, resultClass, params);
        return getFirst(query);
    }

    /**
     * Native Query获取单条数据
     */
    @Override
    public T getBySql(String sql, Class<T> resultClass, Map<String, Object> params) {
    	Query query = getNativeQueryWithParamMap(sql, resultClass, params);
        return getFirst(query);
    }

    /**
     * Native Query列表所有数据获取数据
     */
    @Override
    public <T> List<T> listBySql(String sql, Class<T> resultClass, Object... params) {
        Query query = getNativeQueryWithParams(sql, resultClass, params);
        return query.getResultList();
    }

    /**
     * Native Query列表所有数据获取数据
     */
    @Override
    public <T> List<T> listBySql(String sql, Class<T> resultClass, Map<String, Object> params) {
    	Query query = getNativeQueryWithParamMap(sql, resultClass, params);
        return query.getResultList();
    }
    
    public List<Map<String,Object>> listMapBySql(String sql){
    	
    	Session session = entityManager.unwrap(Session.class);
    	
    	SQLQuery query = session.createSQLQuery(sql);
    	
    	query.setResultTransformer(CriteriaSpecification.ALIAS_TO_ENTITY_MAP);
    	
    	List list = query.list();
    	
    	return list;
    }

    /**
     * Native Query分页获取数据
     */
    @Override
    public PageInfo<T> pageBySql(PageInfo pageInfo, String sql, Class<T> resultClass, Object... params) {
        // 查询数据，统计条数
        Query query = getNativeQueryWithParams(sql, resultClass, params);
        query.setFirstResult((pageInfo.getPageNo() - 1) * pageInfo.getPageSize());
        query.setMaxResults(pageInfo.getPageSize());
        List<T> resultList = query.getResultList();

        Query pageQuery = getNativeQueryWithParams(buildCountHQL(sql), null, params);
        BigDecimal count = (BigDecimal) pageQuery.getSingleResult();
        pageInfo.setList(resultList);
        pageInfo.setCount(count.longValue());
        return pageInfo;
    }

    @Override
    public PageInfo<T> pageBySql(PageInfo pageInfo, String sql, Class<T> resultClass, Map<String, Object> params) {
        // 查询数据，统计条数
        Query query = getNativeQueryWithParamMap(sql, resultClass, params);
        query.setFirstResult((pageInfo.getPageNo() - 1) * pageInfo.getPageSize());
        query.setMaxResults(pageInfo.getPageSize());
        List<T> resultList = query.getResultList();

        Query pageQuery = getNativeQueryWithParamMap(buildCountHQL(sql), null, params);
        BigDecimal count = (BigDecimal) pageQuery.getSingleResult();
        pageInfo.setList(resultList);
        pageInfo.setCount(count.longValue());
        return pageInfo;
    }
}