package com.adc.da.util.utils;

import com.adc.da.base.entity.BaseEntity;
import com.adc.da.util.exception.AdcDaBaseException;
import com.adc.da.util.http.ContentDispositionUtils;
import com.ctc.wstx.util.StringUtil;
import com.ibm.icu.text.CharsetDetector;
import com.ibm.icu.text.CharsetMatch;
import org.apache.commons.io.ByteOrderMark;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.input.BOMInputStream;
import org.apache.poi.ss.usermodel.DateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

/**
 * @Description:CSV控制类导出导入
 * @ModifyBy:
 */
public class CSVUtils {

    /**
     * 日志记录
     */
    private static Logger logger = LoggerFactory.getLogger(CSVUtils.class);
    private static String charaset = "utf-8";

    private static final String NAME = "Content-disposition";
    private static final String VALUE = "attachment;";

    /**
     * 导出
     *
     * @param fileName   必须带有后缀名
     * @param paramsList 所有要生成的参数 一个下标是一行
     * @param response   带有请求头直接流写出
     * @throws IOException
     */
    public static void csvWrite(String fileName, List<String> paramsList,
                                HttpServletResponse response,
                                HttpServletRequest request) throws IOException {
        if (response != null) {
            //设置响应头
            String rtn = ContentDispositionUtils.handler(request, fileName);
            response.addHeader(NAME, VALUE + rtn + ".csv");
            response.setCharacterEncoding(charaset);
            response.setContentType("application/csv;charset=" + charaset);
            ServletOutputStream outputStream = response.getOutputStream();
            File csvFile = File.createTempFile(fileName, ".csv");

            try (FileOutputStream fileOutputStream = new FileOutputStream(csvFile);
                 OutputStreamWriter outputStreamWriter = new OutputStreamWriter(fileOutputStream, charaset)) {
                //生成csv
                outputStreamWriter.write(new String(new byte[]{(byte) 0xEF, (byte) 0xBB, (byte) 0xBF}));
                BufferedWriter bufferedWriter = new BufferedWriter(outputStreamWriter);

                for (int i = 0; i < paramsList.size(); i++) {
                    String row = paramsList.get(i);
                    if (i == paramsList.size() - 1) {
                        bufferedWriter.write((row));
                    } else {
                        bufferedWriter.write((row + ","));
                    }
                }
                bufferedWriter.flush();

                fileOutputStream.flush();
                generatorCsv(outputStream, csvFile);
                response.addHeader("Content-Length", "" + csvFile.length());
            } catch (Exception e) {
                logger.error(e.getMessage());
            }


        }
    }

    /**
     * 写出流
     *
     * @param outputStream
     * @throws IOException
     */
    private static void generatorCsv(OutputStream outputStream, File csvFile) throws IOException {
        try (InputStream in = new FileInputStream(csvFile);) {
            int len = 0;
            byte[] buffer = new byte[1024];
            while ((len = in.read(buffer)) > 0) {
                outputStream.write(buffer, 0, len);
            }
        } catch (FileNotFoundException e ) {
            logger.error(e.getMessage());
        }
    }

    /**
     * 判断文件的编码格式
     *
     * @param
     * @return 文件编码格式
     * @throws Exception
     */
    public static String codeString(InputStream bin) throws IOException {
        int p = (bin.read() << 8) + bin.read();
        String code = null;
       /* switch (p) {
            case 0x7556:
                code = "UTF-8";
                break;
            case 0xfffe:
                code = "Unicode";
                break;
            case 0xfeff:
                code = "UTF-16BE";
                break;
            default:
                code = "UTF-8";
        }*/
        switch (p) {
            case 17263:
                code = "UTF-8";
                break;
            case 0xfffe:
                code = "Unicode";
                break;
            case 0xfeff:
                code = "UTF-16BE";
                break;
            case 21347:
                code = "GBK";
                break;
        }

        return code;
    }

    /**
     * 导入csv 并生成List<LinkedHashMap<String, Object>>
     *
     * @param fieldList 需验证表头信息  一个index 代表一个字段格式-> 中(英文)
     * @param is        输入流
     * @return 如果表里没数据则返回 List->状态isEmpty()
     * @throws IOException
     */
    public static List<LinkedHashMap<String, Object>> csvReader(List<String> fieldList,
                                                                InputStream is,String path) throws IOException {
        List<String> fieldNameList = new ArrayList<>();
        List<LinkedHashMap<String, Object>> returnList = new ArrayList<>();
        int lineTotal = 0;
        File tempFile = new File(path);
        String code = getFileCharsetByICU4J(tempFile);
        BOMInputStream bomIn = new BOMInputStream(is, false, ByteOrderMark.UTF_8, ByteOrderMark.UTF_16LE, ByteOrderMark.UTF_16BE, ByteOrderMark.UTF_32BE, ByteOrderMark.UTF_32LE);
        if(bomIn.hasBOM()){
            code = bomIn.getBOMCharsetName();
        }
        if(code == null){
            code = "GB2312";
        }
        //添加bom头处理
        try (InputStreamReader fileReader = new InputStreamReader(bomIn,code);
             LineNumberReader br = new LineNumberReader(fileReader);) {
            while (br.ready()) {
                String line = br.readLine();
                if(line.startsWith("\"")){
                    line = line.substring(1);
                }
                if(line.endsWith("\"")){
                    line = line.substring(0,line.length()-1);
                }
                lineTotal = br.getLineNumber();
                //分析表头是否符合要求
                if (br.getLineNumber() == 1) {
                    String[] split = line.split(",(?=([^\\\"]*\\\"[^\\\"]*\\\")*[^\\\"]*$)",-1);
//                    if (split.length == 0 || split.length != fieldList.size()) {
//                        throw new AdcDaBaseException("导入文件表头错误，导入失败");
//                    }

                    for (int i = 0; i < split.length; i++) {
                        String tempFieldName = split[i];
                        fieldNameList.add(i, tempFieldName);
                    }
                } else {
                    LinkedHashMap<String, Object> returnMap = new LinkedHashMap<>();
                    //2.往里对应字段加如数据
                    String[] split = line.split(",(?=([^\"]*\"[^\"]*\")*[^\"]*$)",-1);
                    if (split.length > fieldNameList.size()) {
                        throw new AdcDaBaseException("数据填写有误,请检查!");
                    }

                    for (int i = 0; i < split.length; i++) {
                        String value = split[i];
                        returnMap.put(fieldNameList.get(i), value.replace("\"",""));
                    }
                    if (!returnMap.isEmpty()) {
                        returnList.add(returnMap);
                    }
                }
            }
            if (lineTotal<=1){
                throw new AdcDaBaseException("导入数据不能为空,请检查!");
            }

        } catch (IOException | AdcDaBaseException e) {
            throw new AdcDaBaseException(e.getMessage());
        } finally {
            if (is != null) {
                is.close();
            }
        }
        return returnList;
    }







    public static String getFileCharsetByICU4J(File file) {
        String encoding = null;
        try {
            Path path = Paths.get(file.getPath());
            byte[] data = Files.readAllBytes(path);
            CharsetDetector detector = new CharsetDetector();
            detector.setText(data);
            CharsetMatch match = detector.detect();
            if (match == null) {
                return encoding;
            }
            encoding = match.getName();
            if (encoding.equals("ISO-8859-8-I")) {
                encoding = "GBK";
            }
            System.out.println(encoding);
        } catch (IOException var6) {
        }
        return encoding;
    }

   /* public static void main(String[] args) throws Exception {
        InputStreamReader fileReader = new InputStreamReader(new FileInputStream("D:/ScenarioTable.csv"), codeString(new FileInputStream("D:/ScenarioTable.csv")));
        LineNumberReader br = new LineNumberReader(fileReader);
        String line = br.readLine();
        String line2 = br.readLine();
    }*/

    /**
     * 将一行数据转换成 json
     *
     * @param data        一行以逗号分割列的数据
     * @param columnNames 列名
     * @return json
     */
    public static String csv2Json(String data, List<String> columnNames) {
        StringBuilder builder = new StringBuilder("{");
        String[] values = data.split(",");
        int i = 0;
        for (String name :
                columnNames) {
            builder.append("\"").append(name).append("\":").append("\"").append(values[i++]).append("\"").append(",");
        }
        builder.deleteCharAt(builder.lastIndexOf(",")).append("}");
        return builder.toString();
    }

    private static final String KEY = "fileName";

    /**
     * 导出csv 文件
     *
     * @param csvTitleList
     * @param csv2propertyMap
     * @param dataList
     * @param request
     * @param response
     * @param map
     * @throws IOException
     */
    public static void exportCSV(List<String> csvTitleList, Map<String, String> csv2propertyMap, List<? extends BaseEntity> dataList, HttpServletRequest request,
                                 HttpServletResponse response, Map<String, String> map, boolean finish) throws IOException {
        //不用压缩，直接下载
        StringUtils.listToString(csvTitleList);
        ServletOutputStream outputStream = null;
        try {
            //设置响应头
            String rtn = ContentDispositionUtils.handler(request, map.get(KEY));
            response.addHeader(NAME, VALUE + rtn);
            response.setCharacterEncoding(charaset);
            response.setContentType("application/csv;charset=" + charaset);
            outputStream = response.getOutputStream();
            outputStream.flush();

            File csvFile;
            String csvFilePath = map.get("dir") + map.get(KEY);
            if (FileUtil.exists(csvFilePath)) {
                csvFile = org.apache.commons.io.FileUtils.getFile(csvFilePath);
                appendCsvFile(csvTitleList, csv2propertyMap, dataList, map.get("dir"), map.get(KEY));
            } else {
                csvFile = createCsvFile(csvTitleList, csv2propertyMap, dataList, map.get("dir"), map.get(KEY));
            }
            if (finish) {
                generatorCsv(outputStream, csvFile);
                FileUtil.deleteFile(csvFile.getAbsolutePath());
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        } finally {
            if (outputStream != null && finish) {
                outputStream.close();
            }
        }
    }

    /**
     * 导出测试用例csv 文件
     *
     * @param request
     * @param response
     * @param map
     * @throws IOException
     */
    public static void exportCaseCSV(File csvFile, boolean finish, Map<String, String> map, HttpServletResponse response, HttpServletRequest request) throws IOException {
        //不用压缩，直接下载
        ServletOutputStream outputStream = null;
        try {
            //设置响应头
            String rtn = ContentDispositionUtils.handler(request, map.get(KEY));
            response.addHeader(NAME, VALUE + rtn);
            response.setCharacterEncoding(charaset);
            response.setContentType("application/force-download;charset=" + charaset);
            response.addHeader("Content-Length", "" + csvFile.length());
            outputStream = response.getOutputStream();
            outputStream.flush();

            if (finish) {
                generatorCsv(outputStream, csvFile);
                FileUtil.deleteFile(csvFile.getAbsolutePath());
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        } finally {
            if (outputStream != null && finish) {
                outputStream.close();
            }
        }
    }

    public static void saveCSV(List<String> csvTitleList, Map<String, String> csv2propertyMap, List<? extends BaseEntity> dataList,
                               String fileName, String dir) {
        //不用压缩，直接下载
        StringUtils.listToString(csvTitleList);
        try {
            String csvFilePath = dir + fileName;
            if (FileUtil.exists(csvFilePath)) {
                org.apache.commons.io.FileUtils.getFile(csvFilePath);
                appendCsvFile(csvTitleList, csv2propertyMap, dataList, dir, fileName);
            } else {
                createCsvFile(csvTitleList, csv2propertyMap, dataList, dir, fileName);
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        }

    }

    public static File createCsvFile(List<String> csvTitleList, Map<String, String> csv2propertyMap, List<? extends BaseEntity> dataList, String dir, String fileName) {
        FileUtil.mkdir(dir);
        String filePath = dir + fileName;
        File csvFile = new File(filePath);
        try (FileOutputStream fileOutputStream = new FileOutputStream(csvFile);
             OutputStreamWriter outPutStream = new OutputStreamWriter(fileOutputStream, charaset)) {
            //生成csv
            if ("utf-8".equalsIgnoreCase(charaset)) {
                outPutStream.write(new String(new byte[]{(byte) 0xEF, (byte) 0xBB, (byte) 0xBF}));
            }
            BufferedWriter bufferedWriter = new BufferedWriter(outPutStream);
            String title = StringUtils.listToString(csvTitleList);
            bufferedWriter.write(title);
            bufferedWriter.write("\n");
            if (CollectionUtils.isNotEmpty(dataList)) {
                for (BaseEntity baseEntity : dataList) {
                    String row = property2row(baseEntity, csvTitleList, csv2propertyMap);
                    bufferedWriter.write(row);
                    bufferedWriter.write("\n");
                }
            }
            bufferedWriter.flush();
            fileOutputStream.flush();
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
        return csvFile;
    }

    public static File appendCsvFile(List<String> csvTitleList, Map<String, String> csv2propertyMap, List<? extends BaseEntity> dataList, String dir, String fileName) {
        File csvFile = org.apache.commons.io.FileUtils.getFile(dir, fileName);
        if (!csvFile.exists()) {
            throw new AdcDaBaseException("要追加写入文件不存在");
        }
        try (RandomAccessFile randomAccessFile = new RandomAccessFile(csvFile.getAbsolutePath(), "rw")) {
            randomAccessFile.seek(randomAccessFile.length());
            if (CollectionUtils.isNotEmpty(dataList)) {
                for (BaseEntity baseEntity : dataList) {
                    String row = property2row(baseEntity, csvTitleList, csv2propertyMap);
                    randomAccessFile.write(row.getBytes(charaset));
                    randomAccessFile.write("\n".getBytes(charaset));
                }
            }
        } catch (Exception e) {
            logger.error(e.getMessage());
        }
        return csvFile;
    }

    public static void exportCSVZip(HttpServletRequest request, HttpServletResponse response, String
            path, String zipName) throws IOException {
        //空值判断
        if (zipName == null || "".equals(zipName)) {
            //为空设置默认名称
            zipName = "zipData.zip";
        }
        response.reset();
        //设置响应头
        String rtn = ContentDispositionUtils.handler(request, zipName);
//        response.addHeader(NAME, VALUE + rtn);
        response.setCharacterEncoding(charaset);
        response.setContentType("application/force-download;charset=" + charaset);
        response.setHeader("Content-Disposition", "attachment;" + rtn );
        File zipfile = org.apache.commons.io.FileUtils.getFile(path, zipName);
        long size = zipfile.length();
        response.addHeader("Content-Length", "" + size);
        try (ServletOutputStream outputStream = response.getOutputStream()) {
            generatorCsv(outputStream, zipfile);
        } catch (Exception e) {
            throw new AdcDaBaseException(e.getMessage());
        }


    }

    public static void exportCSVZip2(HttpServletRequest request, HttpServletResponse response, String
            path, String zipName) throws IOException {
        //空值判断
        if (zipName == null || "".equals(zipName)) {
            //为空设置默认名称
            zipName = "zipData.zip";
        }

        //设置响应头
        String rtn = ContentDispositionUtils.handler(request, zipName);
        response.addHeader(NAME, VALUE + rtn);
        response.setCharacterEncoding(charaset);
        response.setContentType("application/octet-stream;charset=" + charaset);
        File zipfile = org.apache.commons.io.FileUtils.getFile(path, zipName);
        try (ServletOutputStream outputStream = response.getOutputStream()) {
            generatorCsv(outputStream, zipfile);
        } catch (Exception e) {
            throw new AdcDaBaseException(e.getMessage());
        }

    }

    public static String property2row(BaseEntity baseEntity, List<String> csvTitleList, Map<String, String> csv2propertyMap) {
        Map<String, String> fieldValueMap = ClassUtils.getFieldValueMap(baseEntity);
        StringBuilder row = new StringBuilder();
        //将一个实体拼成csv一行
        for (int i = 0; i < csvTitleList.size(); i++) {
            String title = csvTitleList.get(i);
            String propertyName = csv2propertyMap.get(title);
            String value = fieldValueMap.get(propertyName);
            if (i == csvTitleList.size() - 1) {
                row.append(value);
            } else {
                row.append(value).append(",");
            }
        }
        return row.toString();
    }


    public static void saveCSV1(List<String> csvTitleList, Map<String, String> csv2propertyMap, List<Map<String, String>> dataList,
                                String fileName, String dir, boolean finish) throws IOException {
        //不用压缩，直接下载
        String title = StringUtils.listToString(csvTitleList);
        try {
            File csvFile = null;
            String csvFilePath = dir + fileName;
            if (FileUtil.exists(csvFilePath)) {
                csvFile = FileUtil.getFile(csvFilePath);
                appendCsvFile1(csvTitleList, csv2propertyMap, dataList, dir, fileName);
            } else {
                csvFile = createCsvFile1(csvTitleList, csv2propertyMap, dataList, dir, fileName);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    public static File saveCSVCase(List<String> csvTitleList, List<Map<String, String>> dataList,
                                   String recordID, String dir) throws IOException {
        String fileName = recordID + DateUtils.dateToString(new Date(), "yyyy-MM-dd-HHmmss") + ".csv";
        //不用压缩，直接下载
        try {
            File csvFile = null;
            String csvFilePath = dir + fileName;
            if (FileUtil.exists(csvFilePath)) {
                csvFile = FileUtil.getFile(csvFilePath);
                appendCsvFileCase(csvTitleList, dataList, dir, fileName);
            } else {
                csvFile = createCsvFileCase(csvTitleList, dataList, dir, fileName);
            }
            return csvFile;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static File saveCSVCase2(List<String> csvTitleList, List<Map<String, String>> dataList,
                                   String fileName, String dir) throws IOException {
       // String fileName = recordID + DateUtil.dateToString(new Date(), "yyyy-MM-dd-HHmmss") + ".csv";
        //不用压缩，直接下载
        try {
            File csvFile = null;
            String csvFilePath = dir + fileName;
            if (FileUtil.exists(csvFilePath)) {
                csvFile = FileUtil.getFile(csvFilePath);
                appendCsvFileCase(csvTitleList, dataList, dir, fileName);
            } else {
                csvFile = createCsvFileCase(csvTitleList, dataList, dir, fileName);
            }
            return csvFile;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static File createCsvFile1(List<String> csvTitleList, Map<String, String> csv2propertyMap, List<Map<String, String>> dataList, String dir, String fileName) {
        File csvFile = null;
        FileOutputStream fileOutputStream = null;
        try {
            File tempDir = FileUtil.mkdir(dir);
            String filePath = dir + fileName;
            csvFile = new File(filePath);
            fileOutputStream = new FileOutputStream(csvFile);
            OutputStreamWriter outPutStream = new OutputStreamWriter(fileOutputStream, charaset);
            //生成csv
            if ("utf-8".equalsIgnoreCase(charaset)) {
                outPutStream.write(new String(new byte[]{(byte) 0xEF, (byte) 0xBB, (byte) 0xBF}));
            }
            BufferedWriter bufferedWriter = new BufferedWriter(outPutStream);
            String title = StringUtils.listToString(csvTitleList);
            bufferedWriter.write(title);
            bufferedWriter.write("\n");
            if (CollectionUtils.isNotEmpty(dataList)) {
                for (Map<String, String> baseEntity : dataList) {
                    String row = property2row(baseEntity, csvTitleList, csv2propertyMap);
                    bufferedWriter.write(row);
                    bufferedWriter.write("\n");
                }
            }
            bufferedWriter.flush();
            fileOutputStream.flush();
        } catch (
                IOException e) {
            e.printStackTrace();
        } catch (
                Exception e) {
            e.printStackTrace();
        } finally {
            if (fileOutputStream != null) {
                try {
                    fileOutputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

        }

        return csvFile;
    }

    public static File createCsvFileCase(List<String> csvTitleList, List<Map<String, String>> dataList, String dir, String fileName) {
        File csvFile = null;
        FileOutputStream fileOutputStream = null;
        try {
            File tempDir = FileUtil.mkdir(dir);
            String filePath = dir + fileName;
            csvFile = new File(filePath);
            fileOutputStream = new FileOutputStream(csvFile);
            OutputStreamWriter outPutStream = new OutputStreamWriter(fileOutputStream, charaset);
            //生成csv
            if ("utf-8".equalsIgnoreCase(charaset)) {
                outPutStream.write(new String(new byte[]{(byte) 0xEF, (byte) 0xBB, (byte) 0xBF}));
            }
            BufferedWriter bufferedWriter = new BufferedWriter(outPutStream);
            String title = StringUtils.listToString(csvTitleList);
            bufferedWriter.write(title);
            bufferedWriter.write("\n");
            if (CollectionUtils.isNotEmpty(dataList)) {
                for (Map<String, String> baseEntity : dataList) {
                    String row = property2rowCase(baseEntity, csvTitleList);
                    bufferedWriter.write(row);
                    bufferedWriter.write("\n");
                }
            }
            bufferedWriter.flush();
            fileOutputStream.flush();
        } catch (
                IOException e) {
            e.printStackTrace();
        } catch (
                Exception e) {
            e.printStackTrace();
        } finally {
            if (fileOutputStream != null) {
                try {
                    fileOutputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }

        }

        return csvFile;
    }

    public static File appendCsvFile1(List<String> csvTitleList, Map<String, String> csv2propertyMap, List<Map<String, String>> dataList, String dir, String fileName) {
        File csvFile = FileUtil.getFile(dir, fileName);
        if (!csvFile.exists()) {
            throw new AdcDaBaseException("要追加写入文件不存在");
        }
        RandomAccessFile randomAccessFile = null;
        try {
            randomAccessFile = new RandomAccessFile(csvFile.getAbsolutePath(), "rw");
            randomAccessFile.seek(randomAccessFile.length());
            if (CollectionUtils.isNotEmpty(dataList)) {
                for (Map<String, String> baseEntity : dataList) {
                    String row = property2row(baseEntity, csvTitleList, csv2propertyMap);
                    randomAccessFile.write(row.getBytes(charaset));
                    randomAccessFile.write("\n".getBytes(charaset));
                }
            }
        } catch (
                IOException e) {
            e.printStackTrace();
        } catch (
                Exception e) {
            e.printStackTrace();
        } finally {
            try {
                randomAccessFile.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return csvFile;
    }

    public static File appendCsvFileCase(List<String> csvTitleList, List<Map<String, String>> dataList, String dir, String fileName) {
        File csvFile = FileUtil.getFile(dir, fileName);
        if (!csvFile.exists()) {
            throw new AdcDaBaseException("要追加写入文件不存在");
        }
        RandomAccessFile randomAccessFile = null;
        try {
            randomAccessFile = new RandomAccessFile(csvFile.getAbsolutePath(), "rw");
            randomAccessFile.seek(randomAccessFile.length());
            if (CollectionUtils.isNotEmpty(dataList)) {
                for (Map<String, String> baseEntity : dataList) {
                    String row = property2rowCase(baseEntity, csvTitleList);
                    randomAccessFile.write(row.getBytes(charaset));
                    randomAccessFile.write("\n".getBytes(charaset));
                }
            }
        } catch (
                IOException e) {
            e.printStackTrace();
        } catch (
                Exception e) {
            e.printStackTrace();
        } finally {
            try {
                randomAccessFile.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return csvFile;
    }

    public static String property2row(Map<String, String> fieldValueMap, List<String> csvTitleList, Map<String, String> csv2propertyMap) throws UnsupportedEncodingException {
        StringBuilder row = new StringBuilder();
        //将一个实体拼成csv一行
        for (int i = 0; i < csvTitleList.size(); i++) {
            String title = csvTitleList.get(i);
            String propertyName = csv2propertyMap.get(title);
            String value = fieldValueMap.get(propertyName);
            if (i == csvTitleList.size() - 1) {
                row.append(value);
            } else {
                row.append(value).append(",");
            }
        }
        return row.toString();
    }

    public static String property2rowCase(Map<String, String> fieldValueMap, List<String> csvTitleList) throws UnsupportedEncodingException {
        StringBuilder row = new StringBuilder();
        //将一个实体拼成csv一行
        for (int i = 0; i < csvTitleList.size(); i++) {
            String title = csvTitleList.get(i);
            Object value = fieldValueMap.get(title);
            if (i == csvTitleList.size() - 1) {
                row.append(value);
            } else {
                row.append(value).append(",");
            }
        }
        return row.toString();
    }

}
