package com.adc.da.util.utils;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

/**
 * 反射工具类
 *
 * @author qianqiuxuan
 * date 2019/12/16
 **/
public class ClassUtils {
    private ClassUtils(){}
    /**
     * 取Bean的属性和值对应关系的MAP
     *
     * @param bean
     * @return Map
     */
    public static Map<String, String> getFieldValueMap(Object bean) {
        Class<?> cls = bean.getClass();
        Map<String, String> valueMap = new HashMap<>();
        // 取出bean里的所有方法
        Method[] methods = cls.getDeclaredMethods();
        Field[] fields = cls.getDeclaredFields();

        for (Field field : fields) {
            try {
                String fieldType = field.getType().getSimpleName();
                String fieldGetName = parGetName(field.getName());
                if (!checkGetMet(methods, fieldGetName)) {
                    continue;
                }
                Method fieldGetMet = cls.getMethod(fieldGetName, new Class[]{});
                Object fieldVal = fieldGetMet.invoke(bean, new Object[]{});
                String result = null;

                if ("Date".equals(fieldType)) {
                    if (null != fieldVal) {
                        result = fmtDate((Date) fieldVal);
                    }

                } else if("Double".equals(fieldType)){
                    if(null != fieldVal){
                        NumberFormat nt = NumberFormat.getInstance();
                        nt.setGroupingUsed(false);
                        Double fieldValDb = Double.valueOf((Double)fieldVal);
                        if(fieldValDb == -1111111111.1 || fieldValDb == -1111111111){
                            result = "na";
                        }else{
                            result = nt.format(Double.parseDouble(String.valueOf(fieldVal)));
                        }
                    }

                }else if("Float".equals(fieldType)){
                    if(null != fieldVal){
                        NumberFormat nt = NumberFormat.getInstance();
                        nt.setGroupingUsed(false);
                        Float fieldValFl = Float.valueOf((Float)fieldVal);
                        if(fieldValFl == -1111111111.1 || fieldValFl == -1111111111){
                            result = "na";
                        }else{
                            result = nt.format(Float.parseFloat(String.valueOf(fieldVal)));
                        }
                    }
                }else if("Integer".equals(fieldType)){
                    if(null != fieldVal){
                        NumberFormat nt = NumberFormat.getInstance();
                        nt.setGroupingUsed(false);
                        Integer fieldValInt = Integer.valueOf((Integer)fieldVal);
                        if(fieldValInt == -1111111111){
                            result = "na";
                        }else{
                            result = nt.format(Integer.parseInt(String.valueOf(fieldVal)));
                        }
                    }
                }else{
                    if (null != fieldVal) {
                        if(String.valueOf(fieldVal).equals("-1111111111")){
                            result = "na";
                        }else{
                            result = String.valueOf(fieldVal);
                        }

                    }
                }

                valueMap.put(field.getName(), result);
            } catch (Exception e) {
                continue;
            }
        }
        return valueMap;

    }

    /**
     * set属性的值到Bean
     *
     * @param bean
     * @param valMap
     */
    public static void setFieldValue(Object bean, Map<String, String> valMap) {
        Class<?> cls = bean.getClass();
        // 取出bean里的所有方法
        Method[] methods = cls.getDeclaredMethods();
        Field[] fields = cls.getDeclaredFields();

        for (Field field : fields) {
            try {
                String fieldSetName = parSetName(field.getName());
                if (!checkSetMet(methods, fieldSetName)) {
                    continue;
                }
                Method fieldSetMet = cls.getMethod(fieldSetName, field.getType());
                String value = valMap.get(field.getName());

                if (null != value && !"".equals(value)) {
                    String fieldType = field.getType().getSimpleName();
                    invokeByFieldType(fieldType, bean, fieldSetMet, value);

                }
            } catch (Exception e) {
                System.out.println(e);
                continue;
            }
        }

    }

    private static void invokeByFieldType(String fieldType, Object bean, Method fieldSetMet, String value) {
        try {
            if ("String".equals(fieldType)) {
                fieldSetMet.invoke(bean, value);
            } else if ("Date".equals(fieldType)) {
                Date temp = parseDate(value);
                fieldSetMet.invoke(bean, temp);
            } else if ("Integer".equals(fieldType) || "int".equals(fieldType)) {
                Integer intval = Integer.parseInt(value);
                fieldSetMet.invoke(bean, intval);
            } else if ("Long".equalsIgnoreCase(fieldType)) {
                Long temp = Long.parseLong(value);
                fieldSetMet.invoke(bean, temp);
            } else if ("Double".equalsIgnoreCase(fieldType)) {
                Double temp = Double.parseDouble(value);
                fieldSetMet.invoke(bean, temp);
            }else if ("Float".equalsIgnoreCase(fieldType)) {
                Float temp = Float.parseFloat(value);
                fieldSetMet.invoke(bean, temp);
            }else if ("Boolean".equalsIgnoreCase(fieldType)) {
                Boolean temp = Boolean.parseBoolean(value);
                fieldSetMet.invoke(bean, temp);
            }
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
    }

    /**
     * 格式化string为Date
     *
     * @param datestr
     * @return date
     */
    private static Date parseDate(String datestr) {
        if (null == datestr || "".equals(datestr)) {
            return null;
        }
        try {
            String fmtstr = null;
            if (datestr.contains(":")) {
                fmtstr = "yyyy-MM-dd HH:mm:ss";
            } else {
                fmtstr = "yyyy-MM-dd";
            }
            SimpleDateFormat sdf = new SimpleDateFormat(fmtstr, Locale.UK);
            return sdf.parse(datestr);
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 日期转化为String
     *
     * @param date
     * @return date string
     */
    private static String fmtDate(Date date) {
        if (null == date) {
            return null;
        }
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.US);
            return sdf.format(date);
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * 判断是否存在某属性的 set方法
     *
     * @param methods
     * @param fieldSetMet
     * @return boolean
     */
    private static boolean checkSetMet(Method[] methods, String fieldSetMet) {
        for (Method met : methods) {
            if (fieldSetMet.equals(met.getName())) {
                return true;
            }
        }
        return false;
    }

    /**
     * 判断是否存在某属性的 get方法
     *
     * @param methods
     * @param fieldGetMet
     * @return boolean
     */
    private static boolean checkGetMet(Method[] methods, String fieldGetMet) {
        for (Method met : methods) {
            if (fieldGetMet.equals(met.getName())) {
                return true;
            }
        }
        return false;
    }

    /**
     * 拼接某属性的 get方法
     *
     * @param fieldName
     * @return String
     */
    private static String parGetName(String fieldName) {
        if (null == fieldName || "".equals(fieldName)) {
            return null;
        }
        return "get" + fieldName.substring(0, 1).toUpperCase()
                + fieldName.substring(1);
    }

    /**
     * 拼接在某属性的 set方法
     *
     * @param fieldName
     * @return String
     */
    private static String parSetName(String fieldName) {
        if (null == fieldName || "".equals(fieldName)) {
            return null;
        }
        return "set" + fieldName.substring(0, 1).toUpperCase()
                + fieldName.substring(1);
    }
}
